﻿namespace UltraGridIssue
{
    using System;
    using System.Data;
    using System.Windows.Forms;
    using Infragistics.Win;
    using Infragistics.Win.UltraWinGrid;

    public partial class Form1 : Form
    {
        public Form1()
        {
            this.InitializeComponent();
            this.Load += Form1_Load;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.ultraGrid1.InitializeLayout += UltraGrid1_InitializeLayout;
            this.ultraGrid1.InitializeRow += UltraGrid1_InitializeRow;
            this.ultraGrid1.CellChange += UltraGrid1_CellChange;

            var data = this.GetGridData();
            this.ultraGrid1.SetDataBinding(data, data.TableName);
        }

        private void UltraGrid1_CellChange(object sender, CellEventArgs e)
        {
            int index = 0;

            // Get the value of the ValueList corresponding the new text of the changed cell
            var value = e.Cell.Column.ValueList.GetValue(e.Cell.Text, ref index);

            // Check to which column bellongs the changed cell, and set the value of
            // the cell in the same row in the other column
            if (e.Cell.Column.Key == "Id")
            {
                e.Cell.Row.Cells["Description"].Value = value;
            }

            if (e.Cell.Column.Key == "Description")
            {
                e.Cell.Row.Cells["Id"].Value = value;
            }
        }
        
        private void UltraGrid1_InitializeRow(object sender, InitializeRowEventArgs e)
        {
            // If the row ReInitialize it is not necessary to set its cells value
            // we doing this only once on first initializing of the row
            if (e.ReInitialize)
            {
                return;
            }

            // As the the Description column is unbound we need to populate it
            // with values. On each row we set the value of the Description cell
            // equal to the value of the Id cell
            e.Row.Cells["Description"].Value = e.Row.Cells["Id"].Value;
        }

        private void UltraGrid1_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            // Create two value list and populate them with values from the data table
            var valueListByCode = new ValueList();
            var valueListByDescription = new ValueList();

            var data = this.GetComboBoxData();
            foreach (DataRow row in data.Rows)
            {
                valueListByCode.ValueListItems.Add((int)row[0], row[1].ToString());
                valueListByDescription.ValueListItems.Add((int)row[0], row[2].ToString());
            }

            // Get a reference to the Id column
            var codeColumn = e.Layout.Bands[0].Columns[0];

            // Add one unbound column
            var descriptionCol = e.Layout.Bands[0].Columns.Add("Description");

            // Assign the correct value list
            codeColumn.ValueList = valueListByCode;
            descriptionCol.ValueList = valueListByDescription;

            // Set the columns style accordingly
            codeColumn.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownList;
            descriptionCol.Style = Infragistics.Win.UltraWinGrid.ColumnStyle.DropDownList;

            // Set a new header for the column
            codeColumn.Header.Caption = "Code";

            // Sort the value lists
            valueListByCode.SortStyle = ValueListSortStyle.Ascending;
            valueListByDescription.SortStyle = ValueListSortStyle.Ascending;
        }

        /// <summary>
        /// Create some custom data for the grid
        /// </summary>
        /// <returns>DatatTable filled with data</returns>
        private DataTable GetGridData()
        {
            var dt = new DataTable();

            dt.Columns.Add("Id", typeof(int));

            dt.Rows.Add(1);
            dt.Rows.Add(2);

            return dt;
        }

        /// <summary>
        /// Create some custom data for the drop downs
        /// </summary>
        /// <returns>DataTable filled with data</returns>
        private DataTable GetComboBoxData()
        {
            var dt = new DataTable();

            dt.Columns.Add("Key", typeof(int)).AutoIncrement = true;
            dt.Columns.Add("Code");
            dt.Columns.Add("Description");

            dt.Rows.Add(null, "ABC", "Start of the alphabet");
            dt.Rows.Add(null, "NOP", "Middle of the alphabet");
            dt.Rows.Add(null, "XYZ", "End of the alphabet");

            return dt;
        }
    }
}
